<?php

namespace Drupal\soap_manager\Service;

use Drupal\Core\Config\ConfigFactoryInterface;

/**
 * Provides access to SOAP Endpoint module settings.
 */
class ModuleSettings {

  /**
   * The config factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The settings config object.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * Constructs a ModuleSettings object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   */
  public function __construct(ConfigFactoryInterface $config_factory) {
    $this->configFactory = $config_factory;
    $this->config = $config_factory->get('soap_manager.settings');
  }

  /**
   * Gets the request timeout setting.
   *
   * @return int
   *   The request timeout in seconds.
   */
  public function getRequestTimeout() {
    return $this->config->get('security.request_timeout') ?: 30;
  }

  /**
   * Gets the maximum request size setting.
   *
   * @return int
   *   The maximum request size in kilobytes.
   */
  public function getMaxRequestSize() {
    return $this->config->get('security.max_request_size') ?: 1024;
  }

  /**
   * Checks if rate limiting is enabled.
   *
   * @return bool
   *   TRUE if rate limiting is enabled, FALSE otherwise.
   */
  public function isRateLimitingEnabled() {
    return (bool) $this->config->get('security.rate_limiting');
  }

  /**
   * Gets the requests per hour setting.
   *
   * @return int
   *   The maximum number of requests allowed per hour.
   */
  public function getRequestsPerHour() {
    return $this->config->get('security.requests_per_hour') ?: 100;
  }

  /**
   * Checks if request logging is enabled.
   *
   * @return bool
   *   TRUE if request logging is enabled, FALSE otherwise.
   */
  public function isRequestLoggingEnabled() {
    return (bool) $this->config->get('logging.log_requests');
  }

  /**
   * Checks if response logging is enabled.
   *
   * @return bool
   *   TRUE if response logging is enabled, FALSE otherwise.
   */
  public function isResponseLoggingEnabled() {
    return (bool) $this->config->get('logging.log_responses');
  }

  /**
   * Checks if error logging is enabled.
   *
   * @return bool
   *   TRUE if error logging is enabled, FALSE otherwise.
   */
  public function isErrorLoggingEnabled() {
    return (bool) $this->config->get('logging.log_errors');
  }

}
